# -*- coding: utf-8; -*-
# Copyright (c) 2014-2015 Sebastian Wiesner <swiesner@lunaryorn.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


"""
Visitor functions for nodes.
"""

from . import texinfo
from . import html
from . import latex


def noop(self, node):           # pylint: disable=W0613
    """Do nothing with ``node``."""
    pass


def delegate(target_type):
    """Create visitor functions to delegate the processing of a node.

    ``target_type`` is a type object whose visitor functions shall be used to
    process a node.

    """
    visit = lambda s, n: getattr(s, 'visit_{0}'.format(
        target_type.__name__))(n)
    depart = lambda s, n: getattr(s, 'depart_{0}'.format(
        target_type.__name__))(n)
    return (visit, depart)
